from urllib.request import urlopen
from random import SystemRandom
from string import hexdigits
from datetime import datetime
from urllib.parse import urlencode
from io import BytesIO
from decimal import Decimal

import pandas as pd
import numpy as np

from .country import Country
from const import *


class EUVAT:
    def __init__(self, countries, comments):
        self.countries = countries
        self.comments = comments

    @classmethod
    def from_excel(cls, excel):
        with open("out.xlsx", "wb") as xlsx:
            xlsx.write(excel.read())

        data = pd.read_excel(excel, 1, header=None)[1:]

        COUNTRIES = []
        CATS = []
        COMMENTS = []

        country = None

        for i, row in data.iterrows():
            if i == 1:
                for cell in row[3:]:
                    if cell:
                        CATS.append(cell)
                continue

            if type(row[1]) == str:
                if not row[1].startswith("#"):
                    if country:
                        COUNTRIES.append(country)

                    cdata = row[1].split(" - ")
                    country = Country(cdata[0], cdata[1])
                else:
                    COMMENTS.append(": ".join(row[1].split(": ")[1:]))

            if type(row[2]) == str:
                t = row[2].upper()
                for j, cell in enumerate(row[3:]):
                    if type(cell) == str:
                        rdata = cell.split()
                        rate = Decimal(rdata[0])
                        rcs = []

                        for rc in rdata[1:]:
                            rcs.append(rc.lstrip("(#").rstrip(")"))

                    else:
                        try:
                            assert np.isnan(cell)
                            continue
                        except:
                            rate = Decimal(cell)
                            rcs = []
                            
                    country.add_tax_rate(CATS[j], t, rate, rcs)

        return cls(COUNTRIES, COMMENTS)

    @classmethod
    def from_net(cls, date=datetime.today().date()):
        payload = dict()

        hexval = "".join([SystemRandom().choice([hexdigits])
                         for _ in range(8)])

        payload["result"] = f"eu.europa.ec.taxud.tic.module.utils.vatrates.impl.VatRateSearchResult@{hexval}"

        for i in range(len(ALL_COUNTRIES)):
            payload[f"listOfMsa[{i}.id"] = i + 1

        for i in range(len(CATEGORIES)):
            payload[f"listOfCategories[{i}].id"] = i + 1

        for i, rate in enumerate(RATE_TYPES):
            payload[f"listOfTypes[{i}]"] = rate

        payload["dateFilter"] = date.strftime("%d/%m/%Y")

        xlsx = urlopen(EXPORT_URL, urlencode(payload).encode()).read()

        return cls.from_excel(BytesIO(xlsx))

    def get_tax_rate(self, country, category="E-Services", ratetype="STANDARD", default=Decimal(0), fallback=True):
        for c in self.countries:
            if country in (c.name, c.code):
                if rate := c.get_tax_rate(category, ratetype, fallback):
                    return rate
                break

        return default